@@ -1,24 +1,33 @@
 # coding=utf-8
+# Copyright 2018 Sascha Schirra
 #
-# Copyright 2016 Sascha Schirra
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# This file is part of Ropper.
+# 1. Redistributions of source code must retain the above copyright notice, this
+# list of conditions and the following disclaimer.
 #
-# Ropper is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+# this list of conditions and the following disclaimer in the documentation
+# and/or other materials provided with the distribution.
 #
-# Ropper is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# 3. Neither the name of the copyright holder nor the names of its contributors
+# may be used to endorse or promote products derived from this software without
+# specific prior written permission.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" A ND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 from __future__ import print_function
 from filebytes.pe import ImageDirectoryEntry
-from ropper.common.utils import isHex, toHex, getFileNameFromPath
+from ropper.common.utils import isWindows, isHex, toHex, getFileNameFromPath
 from ropper.common.coloredstring import cstr, Color
 from ropper.common.error import RopperError
 from ropper.loaders.loader import Loader, Type
@@ -28,19 +37,19 @@ from ropper.rop import Ropper, Format
 from ropper.gadget import Gadget, GadgetType
 from binascii import unhexlify
 from codecs import encode, decode
-from hashlib import md5
+from ropper.semantic import Analyser, SemanticInformation
 import tempfile
 import re
 import os
 import multiprocessing
-
-CACHE_FOLDER = 'ropper_cache'
+import sys
 
 def deleteDuplicates(gadgets, callback=None):
     toReturn = []
     inst = set()
     count = 0
     added = False
+    len_gadgets = len(gadgets)
     for i,gadget in enumerate(gadgets):
         inst.add(gadget._gadget)
         if len(inst) > count:
@@ -48,7 +57,7 @@ def deleteDuplicates(gadgets, callback=None):
             toReturn.append(gadget)
             added = True
         if callback:
-            callback(gadget, added, float(i)/(len(gadgets)-1))
+            callback(gadget, added, float(i+1)/(len_gadgets))
             added = False
     return toReturn
 
@@ -71,7 +80,7 @@ def filterBadBytes(gadgets, badbytes, callback=None):
     badbytes = formatBadBytes(badbytes)
     if isinstance(gadgets, dict):
         toReturn = {}
-        
+
         gadget_count = 0
         for file, gadget in gadgets.items():
             gadget_count += len(gadget)
@@ -85,8 +94,8 @@ def filterBadBytes(gadgets, badbytes, callback=None):
                     callback(gadget, added, float(i)/(gadget_count-1))
                     added = False
             toReturn[file] = t
-    elif isinstance(gadgets, list): 
-        toReturn = []  
+    elif isinstance(gadgets, list):
+        toReturn = []
         for i, gadget in enumerate(gadgets):
             if not badbytes or not gadget.addressesContainsBytes(badbytes):
                 toReturn.append(gadget)
@@ -99,23 +108,32 @@ def filterBadBytes(gadgets, badbytes, callback=None):
 
 
 def cfgFilterGadgets(binary, gadgets, callback=None):
-    
+
     def intern(gadgets, callback=None,current=0, length=0):
         result = []
-        
         added = False
-        for gadget in gadgets:
-            loadConfig = binary._binary.dataDirectory[ImageDirectoryEntry.LOAD_CONFIG]
-            if not loadConfig:
-                return gadgets
 
+        loadConfig = binary._binary.dataDirectory[ImageDirectoryEntry.LOAD_CONFIG]
+        if not loadConfig:
+            return gadgets
+        alignedCFGFunctions = [func for func in loadConfig.cfGuardedFunctions if func % 16 == 0]
+
+        # For unaligned functions we can use all the 16-bytes in their aligned space, so we align the unaligned functions
+        unalignedCFGFunctions = [(func - func % 16) for func in loadConfig.cfGuardedFunctions if func % 16 != 0]
+        
+        for gadget in gadgets:
             # calculate relative address of the gadget when loaded to memory
             gadgetRVA = gadget.address - binary.imageBase
 
-            # consider Microsoft CFG implementation imprecision - chop off 3 lsbits
-            gadgetRVA8ByteAligend = gadgetRVA - (gadgetRVA % 8)
-
-            inList = gadgetRVA8ByteAligend in loadConfig.cfGuardedFunctions
+            # consider Microsoft CFG implementation imprecision - chop off 4 lsbits
+            gadgetRVA16ByteAligend = gadgetRVA - (gadgetRVA % 16)
+            
+            # If a guarded function is 16-byte aligned then it's only possible to jump directly to that function.
+            # If a guarded function is not 16-byte aligned, then it's possible to jump to anywhere in the 16-bytes of the aligned function's address
+            if gadgetRVA % 16 == 0:
+                inList = gadgetRVA16ByteAligend in alignedCFGFunctions + unalignedCFGFunctions
+            else:
+                inList = gadgetRVA16ByteAligend in unalignedCFGFunctions
 
             if inList:
                 # this is a gadget which passes CFG checks
@@ -128,7 +146,7 @@ def cfgFilterGadgets(binary, gadgets, callback=None):
                 added = False
             current += 1
         return result
-    
+
     if isinstance(gadgets,  list):
         gadgetLen = len(gadgets)-1
         return intern(gadgets, callback, length=gadgetLen)
@@ -207,6 +225,21 @@ class Options(object):
         elif cfg_only == None:
             options['cfg_only'] = False
 
+        count_of_findings = options.get('count_of_findings')
+        if count_of_findings != None and not isinstance(count_of_findings, int):
+            raise TypeError('cfg_only has to be an instance of int')
+        elif count_of_findings == None:
+            options['count_of_findings'] = 5
+
+        multiprocessing = options.get('multiprocessing')
+        if multiprocessing != None and not isinstance(multiprocessing, bool):
+            raise TypeError('count_of_processes has to be an instance of bool')
+        elif multiprocessing and sys.platform.startswith('win'):
+            raise AttributeError('multiprocessing cannot be used on windows.')
+        elif multiprocessing == None:
+            options['multiprocessing'] = not sys.platform.startswith('win')
+
+
     def items(self):
         for key, value in self.__options_dict.items():
             yield key, value
@@ -237,6 +270,10 @@ class Options(object):
 
 class RopperService(object):
 
+    ROPPER_FOLDER = os.path.expanduser('~') + os.path.sep + ".ropper/"
+    CACHE_FOLDER = os.path.expanduser('~') + os.path.sep + ".ropper/cache/"
+    CACHE_FILE_COUNT = 16
+
     def __init__(self, options={}, callbacks=None):
         super(RopperService, self).__init__()
         self.__options = Options(options, self.__optionChanged)
@@ -253,7 +290,7 @@ class RopperService(object):
     @property
     def ropper(self):
         return self.__ropper
-    
+
     @property
     def options(self):
         return self.__options
@@ -261,16 +298,16 @@ class RopperService(object):
     @property
     def files(self):
         return list(self.__files)
-    
+
     def __optionChanged(self, option, oldvalue, newvalue):
         if hasattr(self, '_%s_changed' % option):
             func = getattr(self, '_%s_changed' % option)
             func(newvalue)
 
     def __prepareGadgets(self, file, gadgets, type=None):
-        
+
         gadgets = self.__filterBadBytes(gadgets)
-        gadgets = self.__filterCfg(file, gadgets, type) 
+        gadgets = self.__filterCfg(file, gadgets, type)
         if not self.__options.all:
             callback = None
             if self.__callbacks and hasattr(self.__callbacks, '__deleteDoubleGadgetsProgress__'):
@@ -295,93 +332,138 @@ class RopperService(object):
         return gadgets
 
     def __getCacheFileName(self, file):
-        m = md5()
-        m.update(file.loader._binary._bytes)
-        d = m.hexdigest()
-        return "%s_%s_%d_%s_%s" % (getFileNameFromPath(file.name), str(file.arch), self.options.inst_count,str(self.options.type), d)
+        return "%s_%s_%d_%s_%d" % (file.loader.checksum, str(file.arch), self.options.inst_count,str(self.options.type), sys.version_info.major)
 
     def __saveCache(self, file):
+        cache_file = None
         try:
-            temp = tempfile.gettempdir() + os.path.sep + CACHE_FOLDER
+            temp = RopperService.CACHE_FOLDER
             if not os.path.exists(temp):
-                os.mkdir(temp)
+                os.makedirs(temp)
 
             cache_file = temp + os.path.sep + self.__getCacheFileName(file)
-
-            if len(file.allGadgets) > 1000:
+            count = RopperService.CACHE_FILE_COUNT
+            if not isWindows() and len(file.allGadgets) > 1000:
                 if os.path.exists(cache_file):
                     os.remove(cache_file)
 
                 length = len(file.allGadgets)
-                count = multiprocessing.cpu_count()+1
+
                 step = int(length / count)
                 for i in range(count-1):
                     gadgets = file.allGadgets[i*step: (i+1)*step]
-                    with open(cache_file+'_%d' % i,'wb') as f:
+                    with open(cache_file+'_%d' % (i+1),'wb') as f:
                         f.write(encode(repr(gadgets).encode('ascii'),'zip'))
 
                 gadgets = file.allGadgets[(count-1)*step:]
-                with open(cache_file+'_%d' % (count-1),'wb') as f:
+                with open(cache_file+'_%d' % (count),'wb') as f:
                     f.write(encode(repr(gadgets).encode('ascii'),'zip'))
                 return
 
             with open(cache_file,'wb') as f:
                 f.write(encode(repr(file.allGadgets).encode('ascii'),'zip'))
         except BaseException as e:
-            raise e
-            if os.path.exists(cache_file):
-                os.remove(cache_file)
+            print(e)
+            if cache_file:
+                for i in range(1, RopperService.CACHE_FILE_COUNT+1):
+                    if os.path.exists(cache_file+'_%d' % i):
+                        os.remove(cache_file+'_%d' % i)
 
 
-    def __loadCachePerProcess(self, cacheFileName, gqueue):
-        with open(cacheFileName,'rb') as f:
-            data = f.read()
-            gqueue.put(eval(decode(data,'zip')))
+    def __loadCachePerProcess(self, fqueue, gqueue):
+        nan=0
+        while True:
+            cacheFileName = fqueue.get()
+            if cacheFileName is None:
+                fqueue.task_done()
+                break
+            if os.path.exists(cacheFileName):
+                with open(cacheFileName,'rb') as f:
+                    data = f.read()
+                    gqueue.put(eval(decode(data,'zip')))
+            else:
+                gqueue.put([])
+            fqueue.task_done()
+
 
     def __loadCache(self, file):
         mp = False
+        nan= 0
         processes = []
-        count = multiprocessing.cpu_count()+1
+        single = False
+        cache_file = None
         try:
-            temp = tempfile.gettempdir() + os.path.sep + CACHE_FOLDER
+            temp = RopperService.CACHE_FOLDER
             cache_file = temp + os.path.sep + self.__getCacheFileName(file)
+
             if not os.path.exists(cache_file):
                 if not os.path.exists(cache_file+'_%d' % 1):
                     return
                 else:
-                    mp = True
-            if not mp:
-                with open(cache_file,'rb') as f:
-                    data = f.read()
-                    return eval(decode(data,'zip'))
+                    if isWindows():
+                        raise RopperError('Cache has to be cleared.')
+                    mp = True and multiprocessing.cpu_count()>1
             else:
-                
+                single = True
+            if self.__callbacks and hasattr(self.__callbacks, '__message__'):
+                self.__callbacks.__message__('Load gadgets from cache')
+            if self.__callbacks and hasattr(self.__callbacks, '__gadgetSearchProgress__'):
+                        self.__callbacks.__gadgetSearchProgress__(None, [], 0)
+            if not mp:
+                all_gadgets = []
+                if single:
+                    with open(cache_file,'rb') as f:
+                        data = f.read()
+                        all_gadgets.extend(eval(decode(data,'zip')))
+                        if self.__callbacks and hasattr(self.__callbacks, '__gadgetSearchProgress__'):
+                            self.__callbacks.__gadgetSearchProgress__(None, all_gadgets, 1.0)
+                else:
+                    for i in range(1,RopperService.CACHE_FILE_COUNT+1):
+                        if os.path.exists(cache_file+'_%d' % i):
+                            with open(cache_file+'_%d' % i,'rb') as f:
+                                data = f.read()
+                                all_gadgets.extend(eval(decode(data,'zip')))
+                                if self.__callbacks and hasattr(self.__callbacks, '__gadgetSearchProgress__'):
+                                    self.__callbacks.__gadgetSearchProgress__(None, all_gadgets, float(i)/RopperService.CACHE_FILE_COUNT)
+                return all_gadgets
+
+            else:
+                count = min(multiprocessing.cpu_count(),RopperService.CACHE_FILE_COUNT)
+
                 gqueue = multiprocessing.Queue()
+                fqueue = multiprocessing.JoinableQueue()
+                for i in range(1,RopperService.CACHE_FILE_COUNT+1):
+                    fqueue.put(cache_file+'_%d' % i)
                 all_gadgets = []
                 for i in range(count):
-                    p=multiprocessing.Process(target=self.__loadCachePerProcess, args=(cache_file+'_%d' % i, gqueue))
+                    p=multiprocessing.Process(target=self.__loadCachePerProcess, args=(fqueue, gqueue))
                     p.start()
                     processes.append(p)
+
                 for i in range(count):
+                    fqueue.put(None)
+
+                for i in range(RopperService.CACHE_FILE_COUNT):
                     gadgets = gqueue.get()
                     all_gadgets.extend(gadgets)
+                    if self.__callbacks and hasattr(self.__callbacks, '__gadgetSearchProgress__'):
+                        self.__callbacks.__gadgetSearchProgress__(None, all_gadgets, float(i+1)/RopperService.CACHE_FILE_COUNT)
+
                 return sorted(all_gadgets, key=Gadget.simpleInstructionString)
         except KeyboardInterrupt:
             if mp:
-                for i in range(count):
-                    p = processes[i]
+                for p in processes:
                     if p and p.is_alive():
                         p.terminate()
-        except:
+        except BaseException as e:
             if mp:
-                for i in range(count):
+                for p in processes:
+                    if p and p.is_alive():
+                        p.terminate()
+            if cache_file:
+                for i in range(1,RopperService.CACHE_FILE_COUNT+1):
                     if os.path.exists(cache_file+'_%d' % i):
                         os.remove(cache_file+'_%d' % i)
-                    p = processes[i]
-                    if p.is_alive():
-                        p.terminate()
-            if os.path.exists(cache_file):
-                os.remove(cache_file)
 
 
     def _badbytes_changed(self, value):
@@ -423,7 +505,7 @@ class RopperService(object):
         return None
 
     def clearCache(self):
-        temp = tempfile.gettempdir() + os.path.sep + CACHE_FOLDER
+        temp = RopperService.CACHE_FOLDER
         if os.path.exists(temp):
             import shutil
             shutil.rmtree(temp)
@@ -431,7 +513,7 @@ class RopperService(object):
 
     def getFileFor(self, name):
         return self._getFileFor(name)
-        
+
     def addFile(self, name, bytes=None, arch=None, raw=False):
         if self._getFileFor(name):
             raise RopperError('file is already added: %s' % name)
@@ -440,8 +522,6 @@ class RopperService(object):
             arch=getArchitecture(arch)
 
         loader = Loader.open(name, bytes=bytes, raw=raw, arch=arch)
-        if len(self.__files) > 0 and self.__files[0].loader.arch != loader.arch:
-            raise RopperError('It is not supported to open file with different architectures! Loaded: %s; File to open: %s' % (str(self.__files[0].loader.arch), str(loader.arch)))
         file = FileContainer(loader)
         self.__files.append(file)
 
@@ -519,11 +599,28 @@ class RopperService(object):
 
         return self.__filterBadBytes(to_return)
 
+    def analyseGadgets(self, fileObject):
+        gadgets = fileObject.gadgets
+        analyser = Analyser()
+        cb = None
+        lg = len(gadgets)
+        if self.__callbacks and hasattr(self.__callbacks, '__analyseGadgetsProgress__'):
+            cb = self.__callbacks.__analyseGadgetsProgress__
+        for i,g in enumerate(gadgets):
+            g.info = analyser.analyse(g)
+            if cb:
+                cb(g, float(i)/lg)
+        if cb:
+             cb(None, 1.0)
+        self.__saveCache(fileObject)
+        fileObject.analysed = True
+
     def loadGadgetsFor(self, name=None):
+
         def load_gadgets(f):
             gtype = None
             cache = False
-            Gadget.IMAGE_BASES[f.name] = f.loader.imageBase
+            Gadget.IMAGE_BASES[f.loader.checksum] = f.loader.imageBase
             if self.options.type == 'rop':
                 gtype = GadgetType.ROP
             elif self.options.type == 'jop':
@@ -531,24 +628,19 @@ class RopperService(object):
             elif self.options.type == 'sys':
                 gtype = GadgetType.SYS
             elif self.options.type == 'all':
-                gtype = GadgetType.ALL    
-            if self.__callbacks and hasattr(self.__callbacks, '__message__'):
-                self.__callbacks.__message__('Try to load gadgets from cache')
+                gtype = GadgetType.ALL
             f.allGadgets = self.__loadCache(f)
             if f.allGadgets == None:
-                if self.__callbacks and hasattr(self.__callbacks, '__message__'):
-                    self.__callbacks.__message__('Could not load gadgets from cache')
                 cache = True
-                f.allGadgets = self.__ropper.searchGadgets(f.loader, instructionCount=self.options.inst_count, gtype=gtype)
-            else:
-                if self.__callbacks and hasattr(self.__callbacks, '__message__'):
-                    self.__callbacks.__message__('Gadgets loaded from cache')
+                f.allGadgets = self.__ropper.searchGadgets(f.loader, instructionCount=self.options.inst_count, gtype=gtype,multiprocessing=self.options.multiprocessing)
+
             if cache:
-                if self.__callbacks and hasattr(self.__callbacks, '__message__'):
-                    self.__callbacks.__message__('Create cache')
                 self.__saveCache(f)
             f.gadgets = self.__prepareGadgets(f, f.allGadgets, f.type)
-         
+            f.analysed = f.gadgets[0].info is not None if len(f.gadgets) > 0 else False
+            #self._analyseGadgets(f.gadgets)
+
+
         if name is None:
             for fc in self.__files:
                 load_gadgets(fc)
@@ -556,7 +648,7 @@ class RopperService(object):
             for fc in self.__files:
                 if fc.loader.fileName == name:
                     load_gadgets(fc)
-                
+
     def printGadgetsFor(self, name=None):
         def print_gadgets(f):
             print(f.loader.fileName)
@@ -608,16 +700,42 @@ class RopperService(object):
             fc = self._getFileFor(name)
             if not fc:
                 raise RopperError('No such file opened: %s' % name)
-            
+
             s = fc.loader.arch.searcher
             for gadget in s.search(fc.gadgets, search, quality):
                     yield(fc.name, gadget)
-        else:        
+        else:
             for fc in self.__files:
                 s = fc.loader.arch.searcher
                 for gadget in s.search(fc.gadgets, search, quality):
                     yield(fc.name, gadget)
 
+    def semanticSearch(self, search, stableRegs=[], name=None):
+        count = 0
+        if name:
+            fc = self._getFileFor(name)
+            if not fc:
+                raise RopperError('No such file opened: %s' % name)
+
+            s = fc.loader.arch.searcher
+            for gadget in s.semanticSearch(fc.gadgets, search, self.options.inst_count, stableRegs):
+                if self.options.count_of_findings == 0 or self.options.count_of_findings > count:
+                    yield(fc.name, gadget)
+                else:
+                    break
+                count += 1
+            self.__saveCache(fc)
+        else:
+            for fc in self.__files:
+                s = fc.loader.arch.searcher
+                for gadget in s.semanticSearch(fc.gadgets, search, self.options.inst_count, stableRegs):
+                    if self.options.count_of_findings == 0 or self.options.count_of_findings > count:
+                        yield(fc.name, gadget)
+                    else:
+                        break
+                    count += 1
+                self.__saveCache(fc)
+
     def searchdict(self, search, quality=None, name=None):
         to_return = {}
         for file, gadget in self.search(search, quality, name):
@@ -642,13 +760,13 @@ class RopperService(object):
                 g = ropper.disassembleAddress(section, fc.loader, address, address - (fc.loader.imageBase+section.offset), length)
                 if not g:
                     raise RopperError('Cannot disassemble address: %s' % toHex(address))
-                    
+
                 if length < 0:
                     length = length * -1
                 return g.disassemblyString()
         return ''
-        
-    def createRopChain(self, chain, options={}):
+
+    def createRopChain(self, chain, arch, options={}):
         callback = None
         if self.__callbacks and hasattr(self.__callbacks, '__ropchainMessages__'):
             callback = self.__callbacks.__ropchainMessages__
@@ -656,9 +774,10 @@ class RopperService(object):
         b = []
         gadgets = {}
         for binary in self.__files:
-            gadgets[binary.loader] = binary.gadgets
-            b.append(binary.loader)
-        generator = RopChain.get(b, gadgets, chain, callback, self.options.badbytes)
+            if str(binary.arch) == arch:
+                gadgets[binary.loader] = binary.gadgets
+                b.append(binary.loader)
+        generator = RopChain.get(b, gadgets, chain, callback, unhexlify(self.options.badbytes))
 
         if not generator:
             raise RopperError('%s does not have support for %s chain generation at the moment. Its a future feature.' % (self.files[0].loader.arch.__class__.__name__, chain))
@@ -670,7 +789,7 @@ class RopperService(object):
         if not file:
             raise RopperError('No such file opened: %s' % name)
         file.loader.imageBase = imagebase
-        Gadget.IMAGE_BASES[name] = file.loader.imageBase
+        Gadget.IMAGE_BASES[file.loader.checksum] = file.loader.imageBase
         if file.loaded and (self.options.badbytes or self.options.cfg_only and file.type == Type.PE):
             file.gadgets = self.__prepareGadgets(file, file.allGadgets, file.type)
 
@@ -704,7 +823,7 @@ class FileContainer(object):
     @property
     def name(self):
         return self.loader.fileName
-    
+
     @property
     def arch(self):
         return self.loader.arch
@@ -712,7 +831,7 @@ class FileContainer(object):
     @property
     def type(self):
         return self.loader.type
-    
+
     @property
     def loaded(self):
         return self.__loaded
@@ -720,7 +839,7 @@ class FileContainer(object):
     @property
     def loader(self):
         return self.__loader
-    
+
     @property
     def gadgets(self):
         return self.__gadgets
@@ -728,7 +847,7 @@ class FileContainer(object):
     @gadgets.setter
     def gadgets(self, gadgets):
         self.__gadgets = gadgets
-    
+
     @property
     def allGadgets(self):
         return self.__all_gadgets
@@ -736,4 +855,4 @@ class FileContainer(object):
     @allGadgets.setter
     def allGadgets(self, gadgets):
         self.__loaded = True if gadgets is not None else False
-        self.__all_gadgets = gadgets
\ No newline at end of file
+        self.__all_gadgets = gadgets
